import _ from "lodash";
/**
 * # Fetch API
 */
export default async function fetchApi(url, options, csrf, 
/**
 * Key to use to grab local Storage csrf value.
 */
localStorageCSRFKey, 
/**
 * Key with which to set the request header csrf
 * value
 */
csrfHeaderKey) {
    let data;
    const csrfKey = "x-dsql-csrf-key";
    const csrfValue = localStorage.getItem(localStorageCSRFKey || csrfKey);
    let finalHeaders = {
        "Content-Type": "application/json",
    };
    if (csrf && csrfValue) {
        finalHeaders[localStorageCSRFKey || csrfKey] = csrfValue;
    }
    if (typeof options === "string") {
        try {
            let fetchData;
            switch (options) {
                case "post":
                    fetchData = await fetch(url, {
                        method: options,
                        headers: finalHeaders,
                    });
                    data = fetchData.json();
                    break;
                default:
                    fetchData = await fetch(url);
                    data = fetchData.json();
                    break;
            }
        }
        catch (error) {
            console.log("FetchAPI error #1:", error.message);
            data = null;
        }
    }
    else if (typeof options === "object") {
        try {
            let fetchData;
            if (options.body && typeof options.body === "object") {
                let oldOptionsBody = _.cloneDeep(options.body);
                options.body = JSON.stringify(oldOptionsBody);
            }
            if (options.headers) {
                options.headers = _.merge(options.headers, finalHeaders);
                const finalOptions = Object.assign({}, options);
                fetchData = await fetch(url, finalOptions);
            }
            else {
                const finalOptions = Object.assign(Object.assign({}, options), { headers: finalHeaders });
                fetchData = await fetch(url, finalOptions);
            }
            data = fetchData.json();
        }
        catch (error) {
            console.log("FetchAPI error #2:", error.message);
            data = null;
        }
    }
    else {
        try {
            let fetchData = await fetch(url);
            data = await fetchData.json();
        }
        catch (error) {
            console.log("FetchAPI error #3:", error.message);
            data = null;
        }
    }
    return data;
}
