import _ from "lodash";
import serverError from "../../backend/serverError";
import runQuery from "../../backend/db/runQuery";
import apiGetGrabQueryAndValues from "../../../utils/grab-query-and-values";
/**
 * # Get Function FOr API
 */
export default async function apiGet({ query, dbFullName, queryValues, tableName, dbSchema, debug, dbContext, forceLocal, }) {
    var _a, _b;
    const queryAndValues = apiGetGrabQueryAndValues({
        query,
        values: queryValues,
    });
    if (typeof query == "string" && query.match(/^alter|^delete|^create/i)) {
        return { success: false, msg: "Wrong Input." };
    }
    let results;
    try {
        let { result, error } = await runQuery({
            dbFullName: dbFullName,
            query: queryAndValues.query,
            queryValuesArray: queryAndValues.values,
            readOnly: true,
            dbSchema,
            tableName,
            dbContext,
            debug,
            forceLocal,
        });
        if (debug && global.DSQL_USE_LOCAL) {
            console.log("apiGet:result", result);
            console.log("apiGet:error", error);
        }
        let tableSchema;
        if (dbSchema) {
            const targetTable = (_a = dbSchema.tables) === null || _a === void 0 ? void 0 : _a.find((table) => table.tableName === tableName);
            if (targetTable) {
                const clonedTargetTable = _.cloneDeep(targetTable);
                delete clonedTargetTable.childTable;
                delete clonedTargetTable.childrenTables;
                delete clonedTargetTable.updateData;
                delete clonedTargetTable.indexes;
                tableSchema = clonedTargetTable;
            }
        }
        if (error)
            throw error;
        if (result.error)
            throw new Error(result.error);
        results = result;
        const resObject = {
            success: true,
            payload: results,
            schema: tableName && tableSchema ? tableSchema : undefined,
        };
        return resObject;
    }
    catch (error) {
        serverError({
            component: "/api/query/get/lines-85-94",
            message: error.message,
        });
        (_b = global.ERROR_CALLBACK) === null || _b === void 0 ? void 0 : _b.call(global, `API Get Error`, error);
        if (debug && global.DSQL_USE_LOCAL) {
            console.log("apiGet:error", error.message);
            console.log("queryAndValues", queryAndValues);
        }
        return {
            success: false,
            payload: null,
            error: error.message,
        };
    }
}
