import parseDbResults from "./parseDbResults";
import serverError from "./serverError";
import grabDSQLConnection from "../../utils/grab-dsql-connection";
import connDbHandler from "../../utils/db/conn-db-handler";
/**
 * # DB handler for specific database
 */
export default async function varDatabaseDbHandler({ queryString, queryValuesArray, database, tableSchema, debug, }) {
    var _a;
    let CONNECTION = grabDSQLConnection({ fa: true });
    if (global.DSQL_USE_LOCAL)
        CONNECTION = grabDSQLConnection({ local: true });
    if (database === null || database === void 0 ? void 0 : database.match(/^datasquirel$/))
        CONNECTION = grabDSQLConnection();
    if (debug) {
        console.log(`varDatabaseDbHandler:query:`, queryString);
        console.log(`varDatabaseDbHandler:values:`, queryValuesArray);
        console.log(`varDatabaseDbHandler:CONNECTION:`, CONNECTION.getConfig());
    }
    let results = await connDbHandler(CONNECTION, queryString, queryValuesArray);
    if (debug) {
        console.log(`varDatabaseDbHandler:results:`, results);
    }
    if (results && tableSchema) {
        try {
            const unparsedResults = results;
            const parsedResults = await parseDbResults({
                unparsedResults: unparsedResults,
                tableSchema: tableSchema,
            });
            return parsedResults;
        }
        catch (error) {
            console.log("\x1b[31mvarDatabaseDbHandler ERROR\x1b[0m =>", database, error);
            (_a = global.ERROR_CALLBACK) === null || _a === void 0 ? void 0 : _a.call(global, `VarDb Handler Error`, error);
            serverError({
                component: "varDatabaseDbHandler/lines-52-53",
                message: error.message,
            });
            return null;
        }
    }
    else if (results) {
        return results;
    }
    else {
        return null;
    }
}
