import fs from "fs";
import grabDirNames from "../../utils/backend/names/grab-dir-names";
import _ from "lodash";
import EJSON from "../../utils/ejson";
import generateTypeDefinition from "./generate-type-definitions";
import { AppNames } from "../../dict/app-names";
export default function dbSchemaToType(params) {
    var _a, _b;
    let datasquirelSchema;
    const { mainShemaJSONFilePath, defaultTableFieldsJSONFilePath } = grabDirNames();
    if (params === null || params === void 0 ? void 0 : params.dbSchema) {
        datasquirelSchema = params.dbSchema;
    }
    else {
        const mainSchema = EJSON.parse(fs.readFileSync(mainShemaJSONFilePath, "utf-8"));
        datasquirelSchema = mainSchema.find((sch) => sch.dbFullName == "datasquirel");
    }
    if (!datasquirelSchema)
        return;
    let tableNames = `export const DsqlTables = [\n${datasquirelSchema.tables
        .map((tbl) => `    "${tbl.tableName}",`)
        .join("\n")}\n] as const`;
    const defaultFields = EJSON.parse(fs.readFileSync(defaultTableFieldsJSONFilePath, "utf-8"));
    const dbTablesSchemas = datasquirelSchema.tables.map((tblSchm) => {
        let newDefaultFields = _.cloneDeep(defaultFields);
        return Object.assign(Object.assign({}, tblSchm), { fields: tblSchm.fields.find((fld) => fld.fieldName == "id")
                ? tblSchm.fields
                : [
                    newDefaultFields.shift(),
                    newDefaultFields.shift(),
                    ...tblSchm.fields,
                    ...newDefaultFields,
                ] });
    });
    const defDbName = (_b = (datasquirelSchema.dbName ||
        ((_a = datasquirelSchema.dbFullName) === null || _a === void 0 ? void 0 : _a.replace(new RegExp(`${AppNames["DsqlDbPrefix"]}\\d+_`), "")))) === null || _b === void 0 ? void 0 : _b.toUpperCase().replace(/ /g, "_");
    const schemas = dbTablesSchemas
        .map((table) => generateTypeDefinition({
        paradigm: "TypeScript",
        table,
        typeDefName: `DSQL_${defDbName}_${table.tableName.toUpperCase()}`,
        allValuesOptional: true,
        addExport: true,
    }))
        .filter((schm) => typeof schm == "string");
    return [tableNames, ...schemas];
}
